<?php

// Token.class.php - Class for manipulating authentication tokens
//
// $Id: Token.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2001-2002 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require_once('DBRow.class.php');

/**
 * Class to assist in using authentication tokens (useful to help avoid 
 * Cross-Site Request Forgery (CSRF) attacks)
 *
 * See http://www.tux.org/~peterw/csrf.txt
 *  
 * Table structure for table 'tokens':
 * 
 * CREATE TABLE tokens (
 *  id varchar(127) NOT NULL default '',
 *  action varchar(64) default NULL,
 *  timestamp int(11) default NULL,
 *  valid_time int(11) default NULL,
 *  PRIMARY KEY  (id)
 * TYPE=MyISAM;
 */
class Token extends DBRow {
	/**
	 * Unique token identifier
	 *
	 * @var string
	 */
	var $id = '';
	
	/**
	 * Permitted action associated with this token
	 *
	 * @var string
	 */
	var $action = '';
	
	/**
	 * UNIX timestamp of token creation
	 *
	 * @var int
	 */
	var $timestamp = 0;
	
	/**
	 * The number of seconds which this token is valid for, from timestamp
	 *
	 * @var int
	 */
	var $valid_time = 0;
	
	var $_Hook_GetDetails_pre = '$this->ExpungeAllExpired();';
	
	function Initialise () {
		$this->id = '';
		$this->action = '';
		$this->timestamp = 0;
		$this->valid_time = 0;
	}
	
	function Generate ($action='any', $valid_time = 3600) {
		$this->ExpungeAllExpired();
		$this->action = $action;
		$this->id = md5(uniqid(rand()));
		$this->timestamp = time();
		$this->valid_time = $valid_time;
		$this->Insert();
		return $this->id;
	}
	
	function Refresh () {
		$this->timestamp = time();
		$this->Update();
	}
	
	function Validate ($action='any', &$error_array) {
		$this->ExpungeAllExpired();
		if ($this->action == $action) {		// If token is valid,
			$this->Delete();								// delete it so it can't be used again
			return true;
		} else {
			$error_array[] = 'Invalid or expired token';
			return false;
		}
	}
	
	function ExpungeAllExpired () {
		// Check if this token is expired
		//if ($this->timestamp+$this->valid_time < time()) $this->Initialise();
		// Delete all tokens older than 60 minutes
		$this->_table->_db->query("DELETE FROM ".$this->_table->_name." WHERE timestamp+valid_time < ".time());
	}
}
